


/**TeX
%----------------------- Auxilliary definitions
\def\dyad#1#2#3{(#1$\otimes$#2 $\rightarrow#3)$}
\def\monad#1#2{(#1$\rightarrow$#2)}
\def\triad#1#2#3#4{(#1$\otimes$#2$\otimes$#3 $\rightarrow#4)$}

\def\dif{{\bf dif}}
\def\proj{{\bf proj}}
\def\join{{\S}}
\def\tuple#1{[ {#1} ]}
\def\card#1{\# #1}
\def\map{{\bf Image}}
\def\reduce#1#2{#1 / #2}
\def\Pr#1#2{{\bf P}_{#1}(#2)}
\def\Pf#1{\Pr{E}{#1} }

\def\sel#1#2{#1{ \@ #2 }}
\def\rept{{\cal WHILE\ }}


 Structured\footnote{Part of Mbase implemented in Java
  Author Paul Cockshott
  Started Aug 1997
  Copyright (c) University of Strathclyde }
  data in the form of
vectors, tuples, sets, relations etc is supported.

     
A partial representation of the lattice of
structured types is given in figure 2.
The crucial principle applied in achieving parallelism is the
provision of higher order functions or functionals which allow
lower level functions to applied to all
elements of  structured values. The meanings of these higher
order functions are discussed in section \ref{Functionals}. The higher order
functions  in
the $\Sigma$ of all structured  types are in addition to 
what they inherit from universal:
$\{ \map \reduce{}{} $  apply cardinality $\}$
For all structured types the implementation provides a method {\tt applytomembers}
that applies a procedure to each element of the structured type.
This iterator can then be used to implement operations like reduce
in a type independent way. At the level of {\tt tstructured} the iterator
is defined as an abstract method that is to be overridden by lower level
methods.

\section{Operations on structured data}
\begin{tabular}{|l|c|r|}\cr
\bf Operator&\bf Type&\bf Explanation\cr
$z\leftarrow\reduce{\circ}{  x}$&\dyad{\dyad{Any}{Any}{Any}}{Structured}{Any}&
\begin{minipage}{2in}if $x=a,b,c$ are
some structured collection and$ \circ$	
a dyadic operator $z=((a\circ b)\circ c)$
\end{minipage}\cr
 $z\leftarrow F \map x$&\dyad{\monad{Any}{Any}}{Structured}{Structured}&
 \begin{minipage}{2in}
   e.g. $F \map \tuple{a,b,c}\rightarrow \tuple{F a, F b , F c}$ see \ref{map}
 \end{minipage}\cr
$z\leftarrow \card x$&\monad{ Struct}{Number}&$z$ is the cardinality of $x$\cr
\end{tabular}


 
 
 */

package strathclyde.cs.relational.structured;
import strathclyde.cs.relational.*;
import strathclyde.cs.relational.operators.*;
import java.util.Enumeration;

public abstract class Structured implements Universal
{	
	
	public	abstract int arity();
	public  abstract double cardinality();
	public abstract Structured image(Moperator op);
	/**TeX form image of set under operator \label{map} */
    
	public Universal reduce(Doperator op)/**TeX reduce the set using a dyadic operator */

	{
		Universal temp;
		Enumeration n=members();
		
		if (n. hasMoreElements())
		{ temp=(Universal)(n.nextElement());
		while(n.hasMoreElements())
			temp=op.apply(temp,(Universal)n.nextElement());
		return temp;
		}
		else return undefinedValue;
	}
	public int hashCode()
		/**TeX
		Hash code formed for structure $S$ with $n$ elements
		\{ S_1, S_2,... S_n\} is given by
		 $H_S(n)$ where

		$$H_S(0) = hash(S_1)$$
		$$H_S(i) = 37 \times H_S(i-1) + hash(S_i)$$	 */
	{
		int temp;
		Enumeration n=members();
		
		if (n. hasMoreElements())
		{ temp=(n.nextElement()).hashCode();
		while(n.hasMoreElements())
			temp=(n.nextElement()).hashCode()+temp*37;
		return temp;
		}
		else return 1;
	}
	public Universal concat(Universal b){ /**TeX defined as + for completeness */
		return plus(b);
	}
    public abstract Enumeration members();
	
	
	
}